<?php

namespace Ulmus\Query;

abstract class Fragment {

    public int $order = 0;

    public abstract function render() /*: mixed*/;

    protected function renderSegments(array $segments, string $glue = " ") : string
    {
        return implode($glue, array_filter($segments, function($i) { return ! is_null($i) && $i !== false && $i !== ""; }));
    }
    
    protected function renderTables(/*array|string*/ $tableList, ? int $maxTableCount = null) : string
    {
        $i = 0;
        $list = [];

        foreach((array) $tableList as $alias => $table) {
            $list[] = ! is_numeric($alias) ? "$table $alias" : $table;
            
            if ( ++$i === $maxTableCount ) {
                break;
            }
        }

        return implode(", ", $list);
    }

    protected function validateFieldType($field) : void
    {
        if ( is_numeric($field) ) {
            throw new \Ulmus\Exception\InvalidFieldType(sprintf("Validation of field `%s` failed from query `%s`", $field, get_class($this)));
        }
    }
}
